import {
  ConflictException,
  Injectable,
  NotFoundException,
} from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { FindOptionsWhere, IsNull, Not, Repository } from 'typeorm';
import { Institucion } from './entity/institucion.entity';
import { Operador } from '../operador/entity/operador.entity';
import { InformacionInstitucionView } from './entity/views/informacion-institucion.view';
import { MinInformacionInstitucionView } from './entity/views/min-informacion-institucion.view';

@Injectable()
export class InstitucionService {
  constructor(
    @InjectRepository(Institucion) private repository: Repository<Institucion>,
    @InjectRepository(InformacionInstitucionView)
    private informacionInstitucionView: Repository<InformacionInstitucionView>,
    @InjectRepository(MinInformacionInstitucionView)
    private minInformacionInstitucionView: Repository<MinInformacionInstitucionView>,
  ) {}

  // Posiblemente desaparezca
  findById(id_institucion: number) {
    return this.repository
      .findOne({ where: { id_institucion } })
      .then((institucion) => {
        if (!institucion)
          throw new NotFoundException('No existe este id institución.');
        return institucion;
      });
  }

  findInfoInstitucionById(id_institucion: number) {
    return this.informacionInstitucionView
      .findOne({ where: { id_institucion } })
      .then((infoInstitucion) => {
        if (!infoInstitucion)
          throw new NotFoundException('No existe este id institución.');
        return this.repository.create({
          id_institucion: infoInstitucion.id_institucion,
        });
      });
  }

  findInfoInstitucionAll() {
    return this.informacionInstitucionView.find().then((infoInstituciones) => {
      const instituciones: Institucion[] = [];

      for (let i = 0; i < infoInstituciones.length; i++)
        instituciones.push(
          this.repository.create({
            id_institucion: infoInstituciones[i].id_institucion,
          }),
        );
      return instituciones;
    });
  }

  findMinInfoInstitucionAll(activo = 0, responsable = false) {
    const busqueda: FindOptionsWhere<MinInformacionInstitucionView> = {};

    if (activo) busqueda.activo = activo;
    if (responsable) busqueda.responsable = Not(IsNull());
    return this.minInformacionInstitucionView
      .find({ where: busqueda })
      .then((infoInstituciones) => {
        const instituciones: Institucion[] = [];

        for (let i = 0; i < infoInstituciones.length; i++)
          instituciones.push(
            this.repository.create({
              id_institucion: infoInstituciones[i].id_institucion,
              activo: infoInstituciones[i].activo === 1,
              correo: infoInstituciones[i].correo,
              dominio: infoInstituciones[i].dominio,
              institucion: infoInstituciones[i].institucion,
              responsable: infoInstituciones[i].responsable,
            }),
          );
        return instituciones;
      });
  }

  update(admin: Operador, attrs: Partial<Institucion>) {
    return this.findInfoInstitucionById(attrs.id_institucion)
      .then((institucion) => {
        if (
          admin.tipoUsuario.id_tipo_usuario === 3 &&
          admin.institucion.id_institucion != institucion.id_institucion
        )
          throw new ConflictException(
            'No puedes actualizar la información de esta institución porque no perteneces a ella.',
          );
        Object.assign(institucion, attrs);
        return this.repository.save(institucion);
      })
      .then((_) => ({
        message: 'Se guardaron los cambios correctamente.',
      }));
  }
}
